@@ -1,13 +1,32 @@
 package HTML::FormFu::Model;
-use Moose;
 
-with 'HTML::FormFu::Role::HasParent';
+use strict;
+use Class::C3;
 
+use HTML::FormFu::Attribute qw( mk_accessors );
 use HTML::FormFu::ObjectUtil qw( form parent );
-use Scalar::Util qw( refaddr reftype );
+use Scalar::Util qw( refaddr );
 use Carp qw( croak );
 
-has type => ( is => 'rw', traits  => ['Chained'] );
+__PACKAGE__->mk_accessors(qw/ type /);
+
+sub new {
+    my $class = shift;
+
+    my %attrs;
+    eval { %attrs = %{ $_[0] } if @_ };
+    croak "attributes argument must be a hashref" if $@;
+
+    my $self = bless {}, $class;
+
+    for my $arg (qw/ parent type /) {
+        croak "$arg attribute required" if !exists $attrs{$arg};
+
+        $self->$arg( $attrs{$arg} );
+    }
+
+    return $self;
+}
 
 sub default_values {
     croak "'default_values' method not implemented by Model class";
@@ -33,8 +52,6 @@ sub clone {
     return bless \%new, ref $self;
 }
 
-__PACKAGE__->meta->make_immutable;
-
 1;
 
 __END__
@@ -57,5 +74,3 @@ Carl Franks, C<cfranks@cpan.org>
 
 This library is free software, you can redistribute it and/or modify it under
 the same terms as Perl itself.
-
-=cut
