@@ -1,16 +1,42 @@
 package HTML::FormFu::Plugin;
-use Moose;
 
-with 'HTML::FormFu::Role::HasParent',
-     'HTML::FormFu::Role::Populate';
+use strict;
+use Class::C3;
 
-use HTML::FormFu::ObjectUtil qw( form parent );
-use Scalar::Util qw( refaddr reftype );
-use Carp qw( croak );
+use HTML::FormFu::Attribute qw/ mk_accessors /;
+use HTML::FormFu::ObjectUtil qw( populate form parent );
+use Scalar::Util qw/ refaddr /;
+use Carp qw/ croak /;
 
-has type => ( is => 'rw', traits  => ['Chained'] );
+use overload
+    'eq' => sub { refaddr $_[0] eq refaddr $_[1] },
+    '==' => sub { refaddr $_[0] eq refaddr $_[1] },
+    bool => sub {1},
+    fallback => 1;
 
-sub pre_process { }
+__PACKAGE__->mk_accessors(qw/ type /);
+
+sub new {
+    my $class = shift;
+
+    my %attrs;
+    eval { %attrs = %{ $_[0] } if @_ };
+    croak "attributes argument must be a hashref" if $@;
+
+    my $self = bless {}, $class;
+
+    for (qw/ type /) {
+        croak "$_ attribute required" if !exists $attrs{$_};
+    }
+
+    if ( exists $attrs{parent} ) {
+        $self->parent( delete $attrs{parent} );
+    }
+
+    $self->populate( \%attrs );
+
+    return $self;
+}
 
 sub process { }
 
@@ -28,8 +54,6 @@ sub clone {
     return bless \%new, ref $self;
 }
 
-__PACKAGE__->meta->make_immutable;
-
 1;
 
 __END__
@@ -58,13 +82,6 @@ is called on any elements.
 
 For field plugins, is called during the field's C<process> call.
 
-=head2 pre_process
-
-For form plugins, is called immediately after C<pre_process>
-is run on the elements.
-
-For element plugins, is called before C<pre_process> is run on form plugins.
-
 =head2 post_process
 
 For form plugins, is called immediately before L<HTML::FormFu/process>
@@ -104,5 +121,3 @@ Carl Franks, C<cfranks@cpan.org>
 
 This library is free software, you can redistribute it and/or modify it under
 the same terms as Perl itself.
-
-=cut
