@@ -4,7 +4,9 @@ use strict;
 use Class::C3;
 
 use HTML::FormFu::Attribute qw( mk_accessors mk_output_accessors );
-use HTML::FormFu::ObjectUtil qw( populate form name parent );
+use HTML::FormFu::ObjectUtil qw(
+    populate form name nested_name nested_names get_nested_hash_value
+    set_nested_hash_value nested_hash_key_exists parent );
 use Scalar::Util qw/ refaddr /;
 use Carp qw/ croak /;
 
