@@ -9,15 +9,15 @@ use Scalar::Util qw/ blessed /;
 use Carp qw/ croak /;
 
 sub process {
-    my ( $self, $values ) = @_;
-
+    my ( $self, $values, $params ) = @_;
+    
     my $return;
     my @errors;
 
     if ( ref $values eq 'ARRAY' ) {
         my @return;
         for my $value (@$values) {
-            my ($return) = eval { $self->transformer($value); };
+            my ($return) = eval { $self->transformer($value, $params); };
             if ($@) {
                 push @errors, $self->return_error($@);
                 push @return, undef;
@@ -29,7 +29,7 @@ sub process {
         $return = \@return;
     }
     else {
-        ($return) = eval { $self->transformer($values); };
+        ($return) = eval { $self->transformer($values, $params); };
         if ($@) {
             push @errors, $self->return_error($@);
         }
